@@ -4,25 +4,26 @@
  * Input of Calma GDS-II stream format.
  * Low-level input.
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  */
 
 #ifndef lint
-static char rcsid[]="$Header: /ufs/repository/magic/calma/CalmaRdio.c,v 1.4 2001/06/15 17:53:20 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/calma/CalmaRdio.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
 #include <sys/types.h>
+#include <time.h>
 
 #include <netinet/in.h>
 
@@ -45,11 +46,14 @@ static char rcsid[]="$Header: /ufs/repository/magic/calma/CalmaRdio.c,v 1.4 2001
 #include "textio/textio.h"
 #include "calma/calmaInt.h"
 
+/* C99 compat */
+#include "calma/calma.h"
+
 /* Forward declarations */
 bool calmaReadR8();
 bool calmaSkipBytes();
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -75,6 +79,7 @@ calmaReadTransform(ptrans, name)
 {
     int nbytes, rtype, flags, angle;
     double dangle;
+    double dmag;
     Transform t;
 
     /* Default is the identity transform */
@@ -100,8 +105,20 @@ calmaReadTransform(ptrans, name)
     if (nbytes < 0) return (FALSE);
     if (rtype == CALMA_MAG)
     {
-	calmaReadError("Magnification ignored for subcell %s\n", name);
-	(void) calmaSkipBytes(nbytes - CALMAHEADERLENGTH);
+	if (nbytes != CALMAHEADERLENGTH + 8)
+	{
+	    (void) calmaSkipBytes(nbytes - CALMAHEADERLENGTH);
+	    return (FALSE);
+	}
+	if (!calmaReadR8(&dmag)) return (FALSE);
+
+        if (dmag != (double)((int)(dmag + 0.5)))
+	{
+	    CalmaReadError("Non-integer magnification (%g) in transform\n", dmag);
+	    CalmaReadError("Rounding to %d.\n", (int)(dmag + 0.5));
+	}
+	GeoScaleTrans(ptrans, (int)(dmag + 0.5), &t);
+	*ptrans = t;
     }
     else UNREADRH(nbytes, rtype);
 
@@ -131,13 +148,13 @@ calmaReadTransform(ptrans, name)
 	case 0:	case 90: case 180: case 270:
 	    break;
 	default:
-	    calmaReadError("Non-Manhattan angle (%d) in transform\n", angle);
+	    CalmaReadError("Non-Manhattan angle (%d) in transform\n", angle);
 	    if (angle < 45) angle = 0;
 	    else if (angle < 135) angle = 90;
 	    else if (angle < 225) angle = 180;
 	    else if (angle < 315) angle = 270;
 	    else angle = 0;
-	    calmaReadError("    Rounding to %d degrees.\n", angle);
+	    CalmaReadError("    Rounding to %d degrees.\n", angle);
     }
 
     /*
@@ -167,7 +184,7 @@ calmaReadTransform(ptrans, name)
 
     return (TRUE);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -205,15 +222,15 @@ calmaReadI2Record(type, pvalue)
 
     /* Read the value */
     READI2(n);
-    if (feof(calmaInputFile)) goto eof;
+    if (FEOF(calmaInputFile)) goto eof;
     *pvalue = n;
     return (TRUE);
 
 eof:
-    calmaReadError("Unexpected EOF.\n");
+    CalmaReadError("Unexpected EOF.\n");
     return (FALSE);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -250,15 +267,90 @@ calmaReadI4Record(type, pvalue)
 
     /* Read the value */
     READI4(n);
-    if (feof(calmaInputFile)) goto eof;
+    if (FEOF(calmaInputFile)) goto eof;
     *pvalue = n;
     return (TRUE);
 
 eof:
-    calmaReadError("Unexpected EOF.\n");
+    CalmaReadError("Unexpected EOF.\n");
     return (FALSE);
 }
-
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * calmaReadStampRecord --
+ *
+ * Read a record that contains a pair of timestamps for creation and
+ * modification dates.
+ *
+ * Results:
+ *	TRUE on success, FALSE if the record type we read is not
+ *	what we're expecting.
+ *
+ * Side effects:
+ *	Consumes input.
+ *	Translates the creation timestamp from GDS format to a standard
+ *	UNIX (time.h) timestamp (seconds since the epoch).
+ *	Stores the result in the integer pointed to by 'stampptr'.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+bool
+calmaReadStampRecord(type, stampptr)
+    int type;
+    int *stampptr;
+{
+    int nbytes, rtype;
+    struct tm gds_timestamp;
+
+    READRH(nbytes, rtype);
+    if (nbytes < 0)
+	goto eof;
+
+    if (type != rtype)
+    {
+	calmaUnexpected(type, rtype);
+	return (FALSE);
+    }
+
+    nbytes -= CALMAHEADERLENGTH;
+    if (nbytes != 24)
+    {
+	/* Not dealing with any timestamp that is not in I2 format */
+	calmaSkipBytes(nbytes);
+	if (stampptr) *stampptr = 0;
+	CalmaReadError("Unknown timestamp format;  setting timestamp to zero.\n");
+	return TRUE;
+    }
+
+    gds_timestamp.tm_wday = 0;  /* Not used by mktime() */
+    gds_timestamp.tm_yday = 0;  /* Not used by mktime() */
+    gds_timestamp.tm_isdst = -1;
+
+    READI2(gds_timestamp.tm_year);
+    READI2(gds_timestamp.tm_mon);
+    READI2(gds_timestamp.tm_mday);
+    READI2(gds_timestamp.tm_hour);
+    READI2(gds_timestamp.tm_min);
+    READI2(gds_timestamp.tm_sec);
+
+    /* GDS timestamps differ from UNIX time structure only by a	*/
+    /* difference of 1 in the month count.			*/
+    gds_timestamp.tm_mon--;
+
+    /* Skip the modification date timestamp */
+    (void) calmaSkipBytes(nbytes - 12);
+
+    if (stampptr) *stampptr = (int)mktime(&gds_timestamp);
+    return (TRUE);
+
+eof:
+    CalmaReadError("Unexpected EOF.\n");
+    return (FALSE);
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -297,17 +389,17 @@ calmaReadStringRecord(type, str)
 
     nbytes -= CALMAHEADERLENGTH;
     *str = (char *) mallocMagic(nbytes + 1);
-    if (fread(*str, sizeof (char), nbytes, calmaInputFile) != nbytes)
+    if (FREAD(*str, sizeof (char), nbytes, calmaInputFile) != nbytes)
 	goto eof;
 
     *(*str + nbytes) = '\0';
     return (TRUE);
 
 eof:
-    calmaReadError("Unexpected EOF.\n");
+    CalmaReadError("Unexpected EOF.\n");
     return (FALSE);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -336,7 +428,7 @@ calmaReadR8(pd)
     double mantissa, d;
     bool isneg;
 
-    if (fread((char *) dchars, sizeof (char), sizeof dchars,
+    if (FREAD((char *) dchars, sizeof (char), sizeof dchars,
 		calmaInputFile) != sizeof dchars)
 	return (FALSE);
 
@@ -374,7 +466,7 @@ calmaReadR8(pd)
     *pd = d;
     return (TRUE);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -420,7 +512,7 @@ skipit:
 	(void) calmaSkipBytes(nbytes - CALMAHEADERLENGTH);
     }
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -465,10 +557,10 @@ calmaSkipExact(type)
     return (TRUE);
 
 eof:
-    calmaReadError("Unexpected EOF.\n");
+    CalmaReadError("Unexpected EOF.\n");
     return (FALSE);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -502,7 +594,7 @@ calmaSkipTo(what)
 
     return (TRUE);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -527,7 +619,7 @@ calmaSkipBytes(nbytes)
     int nbytes;	/* Skip this many bytes */
 {
     while (nbytes-- > 0)
-	if (getc(calmaInputFile) < 0)
+	if (FGETC(calmaInputFile) < 0)
 	    return (FALSE);
 
     return (TRUE);
