@@ -27,7 +27,7 @@ static char rcsid[] __attribute__ ((unused)) ="$Header: /usr/cvsroot/magic-8.0/c
 #include <ctype.h>
 #include <sys/types.h>
 #include <arpa/inet.h>	/* for htons() */
-#ifdef	SYSV
+#if defined(SYSV) || defined(EMSCRIPTEN)
 #include <time.h>
 #else
 #include <sys/time.h>
@@ -50,6 +50,7 @@ static char rcsid[] __attribute__ ((unused)) ="$Header: /usr/cvsroot/magic-8.0/c
 #include "utils/styles.h"
 #include "textio/textio.h"
 #include "calma/calmaInt.h"
+#include "extract/extractInt.h"	/* for LabelList */
 #include "utils/main.h"		/* for Path and CellLibPath */
 #include "utils/stack.h"
 
@@ -117,7 +118,7 @@ typedef struct {
 #define GDS_UNPROCESSED CLIENTDEFAULT
 #define GDS_PROCESSED	1
 
-#define PUSHTILE(tp) \
+#define PUSHTILEC(tp) \
     if ((tp)->ti_client == (ClientData) GDS_UNPROCESSED) { \
 	(tp)->ti_client = (ClientData) GDS_PENDING; \
 	STACKPUSH((ClientData) (tp), SegStack); \
@@ -873,7 +874,8 @@ calmaProcessDef(def, outf, do_library)
 {
     char *filename;
     int polyidx;
-    bool isReadOnly, oldStyle, hasContent, isAbstract, hasGDSEnd;
+    bool isReadOnly, oldStyle, hasContent, isAbstract, hasGDSEnd, needHier;
+    bool hierWrite, arrayWrite;
     HashEntry *he;
 
     /* Skip if already output */
@@ -889,11 +891,8 @@ calmaProcessDef(def, outf, do_library)
 
     /* Read the cell in if it is not already available. */
     if ((def->cd_flags & CDAVAILABLE) == 0)
-    {
-	bool dereference = (def->cd_flags & CDDEREFERENCE) ? TRUE : FALSE;
-	if (!DBCellRead(def, (char *) NULL, TRUE, dereference, NULL))
+	if (!DBCellRead(def, TRUE, TRUE, NULL))
 	    return (0);
-    }
 
     /*
      * Flag an error if attempting to write the default (UNNAMED) cell
@@ -922,6 +921,8 @@ calmaProcessDef(def, outf, do_library)
     DBPropGet(def, "LEFview", &isAbstract);
     DBPropGet(def, "GDS_START", &hasContent);
     DBPropGet(def, "GDS_END", &hasGDSEnd);
+    DBPropGet(def, "CIFhier", &needHier);
+
     filename = (char *)DBPropGet(def, "GDS_FILE", &isReadOnly);
 
     /* When used with "calma addendum true", don't output the read-only	*/
@@ -951,8 +952,29 @@ calmaProcessDef(def, outf, do_library)
      * as they are expected to be in the referenced GDS file.
      */
     if (!hasContent || hasGDSEnd)
-	if (DBCellEnum(def, calmaProcessUse, (ClientData) outf) != 0)
+    {
+	int result;
+
+	if (needHier)
+	{
+	    hierWrite = CIFHierWriteDisable;
+	    arrayWrite = CIFArrayWriteDisable;
+
+	    CIFHierWriteDisable = FALSE;
+	    CIFArrayWriteDisable = FALSE;
+	}
+
+	result = DBCellEnum(def, calmaProcessUse, (ClientData) outf);
+
+	if (needHier)
+	{
+	    CIFHierWriteDisable = hierWrite;
+	    CIFArrayWriteDisable = arrayWrite;
+	}
+
+	if (result != 0)
 	    return 1;
+    }
 
     /* Give some feedback to the user */
     TxPrintf("   Generating output for cell %s\n", def->cd_name);
@@ -1198,11 +1220,52 @@ calmaProcessDef(def, outf, do_library)
     /* Output this cell definition from the Magic database */
     if (!isReadOnly)
 	if (!do_library)
+	{
+	    if (needHier)
+	    {
+		hierWrite = CIFHierWriteDisable;
+		arrayWrite = CIFArrayWriteDisable;
+
+		CIFHierWriteDisable = FALSE;
+		CIFArrayWriteDisable = FALSE;
+	    }
+
 	    calmaOutFunc(def, outf, &TiPlaneRect);
 
+	    if (needHier)
+	    {
+		CIFHierWriteDisable = hierWrite;
+		CIFArrayWriteDisable = arrayWrite;
+	    }
+	}
+
     return 0;
 }
 
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * compport ---
+ *
+ *	Compare two port labels by port index.  Sorting function used with
+ *	qsort().
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+int
+compport(const void *one, const void *two)
+{
+    PortLabel *pl1 = (PortLabel *)one;
+    PortLabel *pl2 = (PortLabel *)two;
+
+    if (pl1->pl_port < pl2->pl_port)
+	return -1;
+    if (pl1->pl_port > pl2->pl_port)
+	return 1;
+    else
+	return 0;
+}
 
 /*
  * ----------------------------------------------------------------------------
@@ -1310,6 +1373,7 @@ calmaOutFunc(def, f, cliprect)
     CIFErrorDef = def;
     CIFGen(def, def, &bigArea, CIFPlanes, &DBAllTypeBits, TRUE, TRUE, FALSE,
 		(ClientData)f);
+
     if (!CIFHierWriteDisable)
 	CIFGenSubcells(def, &bigArea, CIFPlanes);
     if (!CIFArrayWriteDisable)
@@ -1341,7 +1405,8 @@ calmaOutFunc(def, f, cliprect)
 
     if (CalmaDoLabels)
     {
-	int i, ltype, maxport = -1;
+	int i, ltype, numports = 0;
+	LabelList *ll = NULL, *newll;
 
 	for (lab = def->cd_labels; lab; lab = lab->lab_next)
 	{
@@ -1353,23 +1418,42 @@ calmaOutFunc(def, f, cliprect)
 	    }
 	    else
 	    {
-		if ((int)lab->lab_port > maxport)
-		    maxport = (int)lab->lab_port;
+		newll = (LabelList *)mallocMagic(sizeof(LabelList));
+		newll->ll_label = lab;
+		newll->ll_attr = (unsigned int)lab->lab_port;
+		newll->ll_next = ll;
+		ll = newll;
+		numports++;
 	    }
 	}
-	if (maxport >= 0)
-	    for (i = 0; i <= maxport; i++)
-		for (lab = def->cd_labels; lab; lab = lab->lab_next)
-		{
-		    ltype = CIFCurStyle->cs_portText[lab->lab_type];
-		    type = CIFCurStyle->cs_portLayer[lab->lab_type];
-		    if ((type >= 0) && ((lab->lab_flags & PORT_DIR_MASK) != 0) &&
-				(lab->lab_port == i))
-		    {
-			calmaWriteLabelFunc(lab, ltype, type, f);
-			/* break; */  /* Do not limit to unique labels! */
-		    }
-		}
+	if (newll != NULL)
+	{
+	    /* Turn linked list into an array, then run qsort on it	*/
+	    /* to sort by port number.					*/
+
+	    PortLabel *pllist = (PortLabel *)mallocMagic(numports * sizeof(PortLabel));
+	    i = 0;
+	    while (ll != NULL)
+	    {
+		pllist[i].pl_label = ll->ll_label;
+		pllist[i].pl_port = (unsigned int)ll->ll_attr;
+		freeMagic(ll);
+		ll = ll->ll_next;
+		i++;
+	    }	
+
+	    qsort(pllist, numports, sizeof(PortLabel), compport);
+
+	    for (i = 0; i < numports; i++)
+	    {
+		lab = pllist[i].pl_label;
+		ltype = CIFCurStyle->cs_portText[lab->lab_type];
+		type = CIFCurStyle->cs_portLayer[lab->lab_type];
+		if (type >= 0)
+		    calmaWriteLabelFunc(lab, ltype, type, f);
+	    }
+	    freeMagic(pllist);
+	}
     }
 
     /* End of structure */
@@ -2419,7 +2503,7 @@ calmaMergePaintFunc(tile, cos)
     if (SegStack == (Stack *)NULL)
 	SegStack = StackNew(64);
 
-    PUSHTILE(tile);
+    PUSHTILEC(tile);
     while (!StackEmpty(SegStack))
     {
 	t = (Tile *) STACKPOP(SegStack);
@@ -2587,7 +2671,7 @@ calmaMergePaintFunc(tile, cos)
 	    intedges += calmaAddSegment(&lb, is_ext,
 			MIN(RIGHT(t), RIGHT(tp)), TOP(t),
 			MAX(LEFT(t), LEFT(tp)), TOP(t));
-	    if (!is_ext) PUSHTILE(tp);
+	    if (!is_ext) PUSHTILEC(tp);
 	}
 
 	if (split_type == 0x3)
@@ -2607,7 +2691,7 @@ left_search:
 	    intedges += calmaAddSegment(&lb, is_ext,
 			LEFT(t), MIN(TOP(t), TOP(tp)),
 			LEFT(t), MAX(BOTTOM(t), BOTTOM(tp)));
-	    if (!is_ext) PUSHTILE(tp);
+	    if (!is_ext) PUSHTILEC(tp);
 	}
 
 	if (split_type == 0x0)
@@ -2627,7 +2711,7 @@ bottom_search:
 	    intedges += calmaAddSegment(&lb, is_ext,
 			MAX(LEFT(t), LEFT(tp)), BOTTOM(t),
 			MIN(RIGHT(t), RIGHT(tp)), BOTTOM(t));
-	    if (!is_ext) PUSHTILE(tp);
+	    if (!is_ext) PUSHTILEC(tp);
 	}
 
 	if (split_type == 0x1)
@@ -2646,7 +2730,7 @@ right_search:
 	    intedges += calmaAddSegment(&lb, is_ext,
 			RIGHT(t), MAX(BOTTOM(t), BOTTOM(tp)),
 			RIGHT(t), MIN(TOP(t), TOP(tp)));
-	    if (!is_ext) PUSHTILE(tp);
+	    if (!is_ext) PUSHTILEC(tp);
 	}
 
 	/* If tile is isolated, process it now and we're done */
